==> knt_ui-1.2.0.tar.gz: lib/src/widget/knt_rectangle_progress.dart <==
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

import '../../generated/colors.gen.dart';

const _durationProgressingInMillis = Duration(milliseconds: 300);

class KntRectangleProgress extends StatefulWidget {
  final double? progress;
  final double? width;
  final double? height;
  final double? heightFactor;
  final Color? color;
  final Color? backgroundColor;
  final double? borderRadius;
  final Duration? duration;

  const KntRectangleProgress({
    Key? key,
    this.progress,
    this.width,
    this.height,
    this.heightFactor,
    this.color,
    this.backgroundColor,
    this.borderRadius,
    this.duration,
  }) : super(key: key);

  @override
  _KntRectangleProgressState createState() => _KntRectangleProgressState();
}

class _KntRectangleProgressState extends State<KntRectangleProgress>
    with SingleTickerProviderStateMixin {
  late AnimationController _progressController;
  late Animation<double> _animation;
  late double _progress;
  late double _oldProgress;

  @override
  void initState() {
    super.initState();
    _prepareAnimation();
    _runProgress();
  }

  @override
  void didUpdateWidget(KntRectangleProgress oldWidget) {
    super.didUpdateWidget(oldWidget);
    _runProgress();
  }

  @override
  void dispose() {
    _progressController.dispose();
    super.dispose();
  }

  void _prepareAnimation() {
    _progress = widget.progress ?? 0.0;
    _oldProgress = _progress;
    _progressController = AnimationController(
      vsync: this,
      duration: widget.duration ?? _durationProgressingInMillis,
    )..addListener(() => setState(() {}));
    _animation = CurvedAnimation(
      parent: _progressController,
      curve: Curves.ease,
    );
  }

  void _runProgress() {
    // Current progress becomes the old ones
    _oldProgress = _progress;
    // Update to the latest progress
    _progress = widget.progress ?? 0.0;
    if (_oldProgress == _progress) {
      return;
    }
    // Always start animation from the beginning
    if (_progressController.isAnimating || _progressController.isCompleted) {
      _progressController.reset();
    }
    _progressController.forward();
  }

  @override
  Widget build(BuildContext context) {
    final height = widget.height ?? 4.w;
    final heightFactor = widget.heightFactor ?? 1.0;
    final borderRadius = widget.borderRadius ?? 0.w;
    return Container(
      width: widget.width ?? 1.sw,
      height: height,
      decoration: BoxDecoration(
        color: widget.backgroundColor ?? ColorName.dustyGray,
        borderRadius: BorderRadius.circular(borderRadius),
      ),
      child: FractionallySizedBox(
        widthFactor:
            _oldProgress + (_progress - _oldProgress) * _animation.value,
        heightFactor: heightFactor,
        alignment: Alignment.centerLeft,
        child: Container(
          height: height * heightFactor,
          decoration: BoxDecoration(
            color: widget.color ?? ColorName.jungleGreen,
            borderRadius: BorderRadius.circular(borderRadius * heightFactor),
          ),
        ),
      ),
    );
  }
}

==> knt_ui-1.2.2.tar.gz: lib/src/widget/knt_rectangle_progress.dart <==
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

import '../../generated/colors.gen.dart';

class KntRectangleProgress extends StatefulWidget {
  static const durationProgressingInMillis = Duration(milliseconds: 300);
  final double? progress;
  final double? width;
  final double? height;
  final double? heightFactor;
  final bool needShowPercentage;
  final Color? color;
  final Color? backgroundColor;
  final double? borderRadius;
  final Duration? duration;

  /// Only work if [needShowPercentage] is true
  final Widget Function(double)? itemPercentage;

  const KntRectangleProgress({
    Key? key,
    this.progress,
    this.width,
    this.height,
    this.heightFactor,
    this.needShowPercentage = false,
    this.itemPercentage,
    this.color,
    this.backgroundColor,
    this.borderRadius,
    this.duration,
  }) : super(key: key);

  @override
  _KntRectangleProgressState createState() => _KntRectangleProgressState();
}

class _KntRectangleProgressState extends State<KntRectangleProgress>
    with SingleTickerProviderStateMixin {
  late AnimationController _progressController;
  late Animation<double> _animation;
  late double _progress;
  late double _oldProgress;

  @override
  void initState() {
    super.initState();
    _prepareAnimation();
    _runProgress();
  }

  @override
  void didUpdateWidget(KntRectangleProgress oldWidget) {
    super.didUpdateWidget(oldWidget);
    _runProgress();
  }

  @override
  void dispose() {
    _progressController.dispose();
    super.dispose();
  }

  void _prepareAnimation() {
    _progress = widget.progress ?? 0.0;
    _oldProgress = _progress;
    _progressController = AnimationController(
      vsync: this,
      duration: widget.duration ?? KntRectangleProgress.durationProgressingInMillis,
    )..addListener(() => setState(() {}));
    _animation = CurvedAnimation(
      parent: _progressController,
      curve: Curves.ease,
    );
  }

  void _runProgress() {
    // Current progress becomes the old ones
    _oldProgress = _progress;
    // Update to the latest progress
    _progress = widget.progress ?? 0.0;
    if (_oldProgress == _progress) {
      return;
    }
    // Always start animation from the beginning
    if (_progressController.isAnimating || _progressController.isCompleted) {
      _progressController.reset();
    }
    _progressController.forward();
  }

  @override
  Widget build(BuildContext context) {
    final height = widget.height ?? 4.w;
    final heightFactor = widget.heightFactor ?? 1.0;
    final borderRadius = widget.borderRadius ?? 0.w;
    final progress =
        _oldProgress + (_progress - _oldProgress) * _animation.value;
    final progressPresent = '${(progress * 100).toStringAsFixed(0)}%';
    final progressBar = Container(
      width: widget.width ?? 1.sw,
      height: height,
      decoration: BoxDecoration(
        color: widget.backgroundColor ?? ColorName.dustyGray,
        borderRadius: BorderRadius.circular(borderRadius),
      ),
      child: FractionallySizedBox(
        widthFactor: progress,
        heightFactor: heightFactor,
        alignment: Alignment.centerLeft,
        child: Container(
          height: height * heightFactor,
          decoration: BoxDecoration(
            color: widget.color ?? ColorName.jungleGreen,
            borderRadius: BorderRadius.circular(borderRadius * heightFactor),
          ),
        ),
      ),
    );
    if (widget.needShowPercentage) {
      return Column(
        children: [
          widget.itemPercentage != null
              ? widget.itemPercentage!.call(progress)
              : SizedBox(
                  width: 1.sw,
                  child: Text(
                    progressPresent,
                    textAlign: TextAlign.end,
                    style: TextStyle(
                      color: ColorName.doveGray2,
                      fontSize: 14.sp,
                      fontWeight: FontWeight.w500,
                      height: 21.sp / 14.sp,
                    ),
                  ),
                ),
          SizedBox(height: 8.w),
          progressBar,
        ],
      );
    }
    return progressBar;
  }
}
